/**
 * GMyth Library
 *
 * @file gmyth/gmyth_remote_encoder.c
 * 
 * @brief <p> GMythRemoteEncoder class defines functions for playing live tv.
 *
 * The remote encoder is used by gmyth_tvplayer to setup livetv. 
 *
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia.
 * @author Hallyson Luiz de Morais Melo <hallyson.melo@indt.org.br>
 *
 *//*
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gmyth_remote_encoder.h"

#include <assert.h>

#include "gmyth_stringlist.h"

static void gmyth_remote_encoder_class_init          (GMythRemoteEncoderClass *klass);
static void gmyth_remote_encoder_init                (GMythRemoteEncoder *object);

static void gmyth_remote_encoder_dispose  (GObject *object);
static void gmyth_remote_encoder_finalize (GObject *object);

G_DEFINE_TYPE(GMythRemoteEncoder, gmyth_remote_encoder, G_TYPE_OBJECT)
    
static void
gmyth_remote_encoder_class_init (GMythRemoteEncoderClass *klass)
{
    GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;
	
    gobject_class->dispose  = gmyth_remote_encoder_dispose;
    gobject_class->finalize = gmyth_remote_encoder_finalize;	
}

static void
gmyth_remote_encoder_init (GMythRemoteEncoder *gmyth_remote_encoder)
{
}

static void
gmyth_remote_encoder_dispose  (GObject *object)
{
    // GMythRemoteEncoder *gmyth_remote_encoder = GMYTH_REMOTE_ENCODER(object);
    
	G_OBJECT_CLASS (gmyth_remote_encoder_parent_class)->dispose (object);
}


static void
gmyth_remote_encoder_finalize (GObject *object)
{
	g_signal_handlers_destroy (object);

    GMythRemoteEncoder *remote_encoder = GMYTH_REMOTE_ENCODER(object);

	g_debug ("[%s] Closing control socket", __FUNCTION__);
	gmyth_socket_close_connection(remote_encoder->myth_socket);
	g_object_unref (remote_encoder->myth_socket);
    
    G_OBJECT_CLASS (gmyth_remote_encoder_parent_class)->finalize (object);
}

/** Creates a new instance of GMythRemoteEncoder.
 * 
 * @return a new instance of GMythRemoteEncoder.
 */
GMythRemoteEncoder*
gmyth_remote_encoder_new (int num, GString *hostname, gshort port)
{
	GMythRemoteEncoder *encoder = GMYTH_REMOTE_ENCODER ( g_object_new (
			GMYTH_REMOTE_ENCODER_TYPE, FALSE ));
			
	encoder->recorder_num = num;
	encoder->hostname = g_string_new (hostname->str);
	encoder->port = port;
	
	return encoder;
}

/** Configures the remote encoder instance connecting it to Mythtv backend.
 * 
 * @param remote_encoder the GMythRemoteEncoder instance.
 * @return TRUE if successfull, FALSE if any error happens.
 */
gboolean
gmyth_remote_encoder_setup (GMythRemoteEncoder *remote_encoder)
{
	assert (remote_encoder);
	g_debug ("[%s] Creating socket and connecting to backend", __FUNCTION__);

	if (remote_encoder->myth_socket == NULL) { 
		
		remote_encoder->myth_socket = gmyth_socket_new ();
		
		if (!gmyth_socket_connect_to_backend (remote_encoder->myth_socket, remote_encoder->hostname->str, 
					remote_encoder->port, TRUE) ) {
			g_warning ("GMythRemoteEncoder: Connection to backend failed");	
			return FALSE;
		}

	} else {
		g_warning("Remote encoder socket already created\n");
	}

	return TRUE;
}

/** Sends the SPAWN_LIVETV command through Mythtv protocol. This command
 * requests the backend to start capturing TV content.
 * 
 * @param remote_encoder The GMythRemoteEncoder instance.
 * @param tvchain_id The tvchain unique id.
 * @return true if success, false if any error happens.
 */
gboolean
gmyth_remote_encoder_spawntv (GMythRemoteEncoder *remote_encoder, GString *tvchain_id)
{
	GMythStringList *str_list;
	GString *tmp_str;
	
	g_debug ("[%s] Spawntv with tvchain_id = %s", __FUNCTION__, tvchain_id->str);
	
	str_list = gmyth_string_list_new ();
	
	tmp_str = g_string_new ("QUERY_RECORDER ");
	g_string_append_printf (tmp_str, "%d", remote_encoder->recorder_num); 
	
	gmyth_string_list_append_string (str_list, tmp_str);
	gmyth_string_list_append_string (str_list, g_string_new ("SPAWN_LIVETV"));
	gmyth_string_list_append_string (str_list, tvchain_id);
	gmyth_string_list_append_int (str_list, 0); // PIP = FALSE (0)

	gmyth_socket_sendreceive_stringlist (remote_encoder->myth_socket, str_list);
	
    g_string_free (tmp_str, TRUE);
    
    tmp_str = gmyth_string_list_get_string (str_list, 0);
    if (tmp_str == NULL) {
    	g_warning ("[%s] Spawntv request returned %s", __FUNCTION__, tmp_str->str);
    	return FALSE;
	}
	
    if (g_ascii_strncasecmp (tmp_str->str, "ok", 2)) {
    	g_warning ("[%s] Spawntv request returned %s", __FUNCTION__, tmp_str->str);
	    g_object_unref (str_list);
    	return FALSE;
    }

    g_object_unref (str_list);
    return TRUE;

}

/** Sends the command STOP_LIVETV to Mythtv backend.
 * 
 * @param remote_encoder the GMythRemoteEncoder instance.
 * @return true if success, false if any error happens.
 */
gboolean
gmyth_remote_encoder_stop_livetv (GMythRemoteEncoder *remote_encoder)
{
	GMythStringList *str_list;
	GString *tmp_str;

	g_debug ("[%s]", __FUNCTION__);

	str_list = gmyth_string_list_new ();

	tmp_str = g_string_new ("QUERY_RECORDER ");
	g_string_append_printf (tmp_str, "%d", remote_encoder->recorder_num); 
	gmyth_string_list_append_string (str_list, g_string_new ("STOP_LIVETV"));

	gmyth_socket_sendreceive_stringlist (remote_encoder->myth_socket, str_list);

	g_string_free (tmp_str, TRUE);

	tmp_str = gmyth_string_list_get_string (str_list, 0);
	if (g_ascii_strncasecmp (tmp_str->str, "ok", 2)) {
		g_warning ("[%s] Stop livetv request returned %s", __FUNCTION__, tmp_str->str);
		g_object_unref (str_list);
		return FALSE;
	}

	g_object_unref (str_list);
	return TRUE;

}
